[systemd-commits] 2 commits - src/shared src/systemctl src/test

Zbigniew Jędrzejewski-Szmek zbyszek at kemper.freedesktop.org
Sun Dec 22 15:49:21 PST 2013


 src/shared/cgroup-util.c    |   45 +++++++++++++++++++++++++++++++++++---------
 src/systemctl/systemctl.c   |    6 +++--
 src/test/test-cgroup-util.c |   19 +++++++++++++++---
 3 files changed, 56 insertions(+), 14 deletions(-)

New commits:
commit d4fffc4b8beb86e77fd710c1f43913a490ed083a
Author: Zbigniew Jędrzejewski-Szmek <zbyszek at in.waw.pl>
Date:   Sun Dec 22 18:28:03 2013 -0500

    Fix extraction of _SYSTEMD_USER_UNIT
    
    Units from user services underneath user at .service would not be detected
    properly.

diff --git a/src/test/test-cgroup-util.c b/src/test/test-cgroup-util.c
index eae6739..f94f213 100644
--- a/src/test/test-cgroup-util.c
+++ b/src/test/test-cgroup-util.c
@@ -27,8 +27,11 @@
 
 static void check_p_d_u(const char *path, int code, const char *result) {
         _cleanup_free_ char *unit = NULL;
+        int r;
 
-        assert_se(cg_path_decode_unit(path, &unit) == code);
+        r = cg_path_decode_unit(path, &unit);
+        printf("%s: %s → %s %d expected %s %d\n", __func__, path, unit, r, result, code);
+        assert_se(r == code);
         assert_se(streq_ptr(unit, result));
 }
 
@@ -46,8 +49,11 @@ static void test_path_decode_unit(void) {
 
 static void check_p_g_u(const char *path, int code, const char *result) {
         _cleanup_free_ char *unit = NULL;
+        int r;
 
-        assert_se(cg_path_get_unit(path, &unit) == code);
+        r = cg_path_get_unit(path, &unit);
+        printf("%s: %s → %s %d expected %s %d\n", __func__, path, unit, r, result, code);
+        assert_se(r == code);
         assert_se(streq_ptr(unit, result));
 }
 
@@ -61,12 +67,17 @@ static void test_path_get_unit(void) {
         check_p_g_u("/system.slice/getty####@tty6.service/xxx", -EINVAL, NULL);
         check_p_g_u("/system.slice/system-waldo.slice/foobar.service/sdfdsaf", 0, "foobar.service");
         check_p_g_u("/system.slice/system-waldo.slice/_cpu.service/sdfdsaf", 0, "cpu.service");
+        check_p_g_u("/user.slice/user-1000.slice/user at 1000.service/server.service", 0, "user at 1000.service");
+        check_p_g_u("/user.slice/user-1000.slice/user at .service/server.service", -EINVAL, NULL);
 }
 
 static void check_p_g_u_u(const char *path, int code, const char *result) {
         _cleanup_free_ char *unit = NULL;
+        int r;
 
-        assert_se(cg_path_get_user_unit(path, &unit) == code);
+        r = cg_path_get_user_unit(path, &unit);
+        printf("%s: %s → %s %d expected %s %d\n", __func__, path, unit, r, result, code);
+        assert_se(r == code);
         assert_se(streq_ptr(unit, result));
 }
 
@@ -81,6 +92,8 @@ static void test_path_get_user_unit(void) {
         check_p_g_u_u("/xyz.slice/xyz-waldo.slice/session-77.scope/foobar at pie.service/pa/po", 0, "foobar at pie.service");
         check_p_g_u_u("/meh.service", -ENOENT, NULL);
         check_p_g_u_u("/session-3.scope/_cpu.service", 0, "cpu.service");
+        check_p_g_u_u("/user.slice/user-1000.slice/user at 1000.service/server.service", 0, "server.service");
+        check_p_g_u_u("/user.slice/user-1000.slice/user at .service/server.service", -ENOENT, NULL);
 }
 
 static void check_p_g_s(const char *path, int code, const char *result) {

commit 58684be9a781be9797142bce442bbac6fb6a7738
Author: Zbigniew Jędrzejewski-Szmek <zbyszek at in.waw.pl>
Date:   Sun Dec 22 16:11:45 2013 -0500

    systemctl: also color filenames of drop-ins in cat

diff --git a/src/systemctl/systemctl.c b/src/systemctl/systemctl.c
index d0a03d9..c20a82b 100644
--- a/src/systemctl/systemctl.c
+++ b/src/systemctl/systemctl.c
@@ -3823,9 +3823,11 @@ static int cat(sd_bus *bus, char **args) {
                 }
 
                 STRV_FOREACH(path, dropin_paths) {
-                        printf("%s# %s\n",
+                        printf("%s%s# %s%s\n",
                                isempty(fragment_path) && path == dropin_paths ? "" : "\n",
-                               *path);
+                               ansi_highlight_blue(),
+                               *path,
+                               ansi_highlight_off());
                         fflush(stdout);
 
                         r = sendfile_full(STDOUT_FILENO, *path);



More information about the systemd-commits mailing list
diff --git a/src/shared/cgroup-util.c b/src/shared/cgroup-util.c
index 309f65d..74889b6 100644
--- a/src/shared/cgroup-util.c
+++ b/src/shared/cgroup-util.c
@@ -1183,6 +1183,9 @@ int cg_pid_get_unit(pid_t pid, char **unit) {
         return cg_path_get_unit(cgroup, unit);
 }
 
+/**
+ * Skip session-*.scope, but require it to be there.
+ */
 static const char *skip_session(const char *p) {
         size_t n;
 
@@ -1191,7 +1194,27 @@ static const char *skip_session(const char *p) {
         p += strspn(p, "/");
 
         n = strcspn(p, "/");
-        if (n <= 12 || memcmp(p, "session-", 8) != 0 || memcmp(p + n - 6, ".scope", 6) != 0)
+        if (n < strlen("session-x.scope") || memcmp(p, "session-", 8) != 0 || memcmp(p + n - 6, ".scope", 6) != 0)
+                return NULL;
+
+        p += n;
+        p += strspn(p, "/");
+
+        return p;
+}
+
+/**
+ * Skip user@*.service, but require it to be there.
+ */
+static const char *skip_user_manager(const char *p) {
+        size_t n;
+
+        assert(p);
+
+        p += strspn(p, "/");
+
+        n = strcspn(p, "/");
+        if (n < strlen("user at x.service") || memcmp(p, "user@", 5) != 0 || memcmp(p + n - 8, ".service", 8) != 0)
                 return NULL;
 
         p += n;
@@ -1201,7 +1224,7 @@ static const char *skip_session(const char *p) {
 }
 
 int cg_path_get_user_unit(const char *path, char **unit) {
-        const char *e;
+        const char *e, *t;
 
         assert(path);
         assert(unit);
@@ -1213,13 +1236,17 @@ int cg_path_get_user_unit(const char *path, char **unit) {
         /* Skip slices, if there are any */
         e = skip_slices(path);
 
-        /* Skip the session scope, require that there is one */
-        e = skip_session(e);
-        if (!e)
-                return -ENOENT;
-
-        /* And skip more slices */
-        e = skip_slices(e);
+        /* Skip the session scope... */
+        t = skip_session(e);
+        if (t)
+                /* ... and skip more slices if there's one */
+                e = skip_slices(t);
+        else {
+                /* ... or require a user manager unit to be there */
+                e = skip_user_manager(e);
+                if (!e)
+                        return -ENOENT;
+        }
 
         return cg_path_decode_unit(e, unit);
 }